@@ -49,6 +49,8 @@
 #define HA_OPEN_MERGE_TABLE		2048U
 #define HA_OPEN_FOR_CREATE              4096U
 #define HA_OPEN_FOR_DROP                (1U << 13) /* Open part of drop */
+#define HA_OPEN_GLOBAL_TMP_TABLE	(1U << 14) /* TMP table used by repliction */
+#define HA_OPEN_SIZE_TRACKING           (1U << 15)
 
 /*
   Allow opening even if table is incompatible as this is for ALTER TABLE which
@@ -97,16 +99,18 @@ enum ha_rkey_function {
   HA_READ_MBR_EQUAL
 };
 
-	/* Key algorithm types */
+/* Key algorithm types (stored in .frm) */
 
 enum ha_key_alg {
-  HA_KEY_ALG_UNDEF=	0,		/* Not specified (old file) */
-  HA_KEY_ALG_BTREE=	1,		/* B-tree, default one          */
-  HA_KEY_ALG_RTREE=	2,		/* R-tree, for spatial searches */
-  HA_KEY_ALG_HASH=	3,		/* HASH keys (HEAP tables) */
-  HA_KEY_ALG_FULLTEXT=	4,		/* FULLTEXT (MyISAM tables) */
-  HA_KEY_ALG_LONG_HASH= 5,		/* long BLOB keys */
-  HA_KEY_ALG_UNIQUE_HASH= 6		/* Internal UNIQUE hash (Aria) */
+  HA_KEY_ALG_UNDEF=       0,            /* Not specified. Practically the
+                                           key will be B-tree or hash   */
+  HA_KEY_ALG_BTREE=       1,            /* B-tree, default one          */
+  HA_KEY_ALG_RTREE=       2,            /* R-tree, for spatial searches */
+  HA_KEY_ALG_HASH=        3,            /* HASH keys (HEAP tables)      */
+  HA_KEY_ALG_FULLTEXT=    4,            /* FULLTEXT                     */
+  HA_KEY_ALG_LONG_HASH=   5,            /* long BLOB keys               */
+  HA_KEY_ALG_UNIQUE_HASH= 6,            /* Internal UNIQUE hash (Aria)  */
+  HA_KEY_ALG_VECTOR=      7             /* Vector search index          */
 };
 
         /* Storage media types */ 
@@ -219,7 +223,10 @@ enum ha_extra_function {
   /** Start writing rows during ALTER TABLE...ALGORITHM=COPY. */
   HA_EXTRA_BEGIN_ALTER_COPY,
   /** Finish writing rows during ALTER TABLE...ALGORITHM=COPY. */
-  HA_EXTRA_END_ALTER_COPY
+  HA_EXTRA_END_ALTER_COPY,
+  /** Abort of writing rows during ALTER TABLE..ALGORITHM=COPY or
+  CREATE..SELCT */
+  HA_EXTRA_ABORT_ALTER_COPY
 };
 
 /* Compatible option, to be deleted in 6.0 */
@@ -267,51 +274,49 @@ enum ha_base_keytype {
   Note that these can only be up to 16 bits!
 */
 
-#define HA_NOSAME		 1U	/* Set if not dupplicated records */
-#define HA_PACK_KEY		 2U	/* Pack string key to previous key */
-#define HA_AUTO_KEY		 16U    /* MEMORY/MyISAM/Aria internal */
-#define HA_BINARY_PACK_KEY	 32U	/* Packing of all keys to prev key */
-#define HA_FULLTEXT		128U    /* For full-text search */
-#define HA_SPATIAL		1024U   /* For spatial search */
-#define HA_NULL_ARE_EQUAL	2048U	/* NULL in key are cmp as equal */
-#define HA_GENERATED_KEY	8192U	/* Automatically generated key */
-/* 
-  Part of unique hash key. Used only for temporary (work) tables so is not
-  written to .frm files.
-*/
-#define HA_UNIQUE_HASH          262144U
-
-        /* The combination of the above can be used for key type comparison. */
-#define HA_KEYFLAG_MASK (HA_NOSAME | HA_AUTO_KEY | \
-                         HA_FULLTEXT | \
-                         HA_SPATIAL | HA_NULL_ARE_EQUAL | HA_GENERATED_KEY | \
-                         HA_UNIQUE_HASH)
+#define HA_NOSAME                  1U   /* Set if not dupplicated records */
+#define HA_PACK_KEY                2U   /* Pack string key to previous key */
+#define HA_SPACE_PACK_USED         4U   /* Test for if SPACE_PACK used */
+#define HA_VAR_LENGTH_KEY          8U   /* automatic bit */
+#define HA_AUTO_KEY               16U   /* MEMORY/MyISAM/Aria internal */
+#define HA_BINARY_PACK_KEY        32U   /* Packing of all keys to prev key */
+#define HA_NULL_PART_KEY          64U   /* automatic bit */
+#define HA_FULLTEXT_legacy       128U   /* For full-text search */
+#define HA_SORT_ALLOWS_SAME      512U   /* Intern bit when sorting records */
+#define HA_SPATIAL_legacy       1024U   /* For spatial search */
+#define HA_NULL_ARE_EQUAL       2048U   /* NULL in key are cmp as equal */
+#define HA_USES_COMMENT         4096U   /* automatic bit */
+#define HA_GENERATED_KEY        8192U   /* Automatically generated key */
+#define HA_USES_PARSER         16384U   /* Fulltext index uses [pre]parser */
+#define HA_USES_BLOCK_SIZE     32768U   /* automatic bit */
 
 /*
   Key contains partial segments.
 
   This flag is internal to the MySQL server by design. It is not supposed
   neither to be saved in FRM-files, nor to be passed to storage engines.
-  It is intended to pass information into internal static sort_keys(KEY *,
-  KEY *) function.
+  It is intended to pass information into internal sort_keys(KEY *, KEY *)
+  function.
 
   This flag can be calculated -- it's based on key lengths comparison.
 */
-#define HA_KEY_HAS_PART_KEY_SEG 65536
-/* Internal Flag Can be calculated */
-#define HA_INVISIBLE_KEY 2<<18
-	/* Automatic bits in key-flag */
-
-#define HA_SPACE_PACK_USED	 4	/* Test for if SPACE_PACK used */
-#define HA_VAR_LENGTH_KEY	 8
-#define HA_NULL_PART_KEY	 64
-#define HA_USES_COMMENT          4096
-#define HA_USES_PARSER           16384  /* Fulltext index uses [pre]parser */
-#define HA_USES_BLOCK_SIZE	 ((uint) 32768)
-#define HA_SORT_ALLOWS_SAME      512    /* Intern bit when sorting records */
+#define HA_KEY_HAS_PART_KEY_SEG 65536U
 
 /* This flag can be used only in KEY::ext_key_flags */
-#define HA_EXT_NOSAME            131072
+#define HA_EXT_NOSAME         131072U
+
+/*
+  Part of unique hash key. Used only for temporary (work) tables so is not
+  written to .frm files.
+*/
+#define HA_UNIQUE_HASH        262144U
+
+/* Internal Flag Can be calculated */
+#define HA_INVISIBLE_KEY     (2<<18)
+
+/* The combination of the above can be used for key type comparison. */
+#define HA_KEYFLAG_MASK (HA_NOSAME | HA_AUTO_KEY | HA_NULL_ARE_EQUAL | \
+                         HA_GENERATED_KEY | HA_UNIQUE_HASH)
 
 	/* These flags can be added to key-seg-flag */
 
@@ -377,6 +382,12 @@ enum ha_base_keytype {
 #define HA_CREATE_INTERNAL_TABLE 256U
 #define HA_PRESERVE_INSERT_ORDER 512U
 #define HA_CREATE_NO_ROLLBACK    1024U
+/*
+  A temporary table that can be used by different threads, eg. replication
+  threads. This flag ensure that memory is not allocated with THREAD_SPECIFIC,
+  as we do for other temporary tables.
+*/
+#define HA_CREATE_GLOBAL_TMP_TABLE 2048U
 
 /* Flags used by start_bulk_insert */
 
@@ -455,7 +466,7 @@ enum ha_base_keytype {
 #define HA_ERR_RETRY_INIT 129 /* Initialization failed and should be retried */
 #define HA_ERR_NOT_A_TABLE      130     /* not a MYI file - no signature */
 #define HA_ERR_WRONG_COMMAND	131	/* Command not supported */
-#define HA_ERR_OLD_FILE		132	/* old databasfile */
+#define HA_ERR_OLD_FILE		132	/* old database file */
 #define HA_ERR_NO_ACTIVE_RECORD 133	/* No record read in update() */
 #define HA_ERR_RECORD_DELETED	134	/* A record is not there */
 #define HA_ERR_RECORD_FILE_FULL 135	/* No more room in file */
@@ -536,7 +547,10 @@ enum ha_base_keytype {
 #define HA_ERR_COMMIT_ERROR       197
 #define HA_ERR_PARTITION_LIST     198
 #define HA_ERR_NO_ENCRYPTION      199
-#define HA_ERR_LAST               199  /* Copy of last error nr * */
+#define HA_ERR_ROLLBACK           200  /* Automatic rollback done */
+#define HA_ERR_LOCAL_TMP_SPACE_FULL   201
+#define HA_ERR_GLOBAL_TMP_SPACE_FULL  202
+#define HA_ERR_LAST               202  /* Copy of last error nr * */
 
 /* Number of different errors */
 #define HA_ERR_ERRORS            (HA_ERR_LAST - HA_ERR_FIRST + 1)
@@ -666,13 +680,8 @@ typedef struct st_page_range
 #define unused_page_range { UNUSED_PAGE_NO, UNUSED_PAGE_NO }
 
 /* For number of records */
-#ifdef BIG_TABLES
 #define rows2double(A)	ulonglong2double(A)
 typedef my_off_t	ha_rows;
-#else
-#define rows2double(A)	(double) (A)
-typedef ulong		ha_rows;
-#endif
 
 #define HA_POS_ERROR	(~ (ha_rows) 0)
 #define HA_OFFSET_ERROR	(~ (my_off_t) 0)
